<?xml version="1.0" encoding="ascii"?>

<xsl:stylesheet version="1.1"
		xmlns:xsl="http://www.w3.org/1999/XSL/Transform">

  <xsl:preserve-space elements="coc-quotes"/>
  <xsl:preserve-space elements="code"/>

<!--xslt document for creating the GUI-man directory for CoCoA-->

  <xsl:output method="text"/>
  <xsl:strip-space elements="*"/>	

  <xsl:template match="help">

<!-- ____ COMMANDS ______________________________________________ -->

<xsl:variable name="file_cmd" select="'commands.html'"/>
<xsl:document href="{$file_cmd}">

&lt;HTML&gt;
&lt;HEAD&gt;
&lt;link REL="stylesheet" TYPE="text/css" href="gui.css"&gt;
&lt;TITLE&gt;
  <xsl:apply-templates select="title"/>
&lt;/TITLE&gt;
&lt;/HEAD&gt;

&lt;BODY&gt;

&lt;DIV&gt;

  <xsl:for-each select="cocoa_commands/command">
    <xsl:sort select="title"/>
    <xsl:variable name="pos_num" select="position()" data-type="number"/>
    <xsl:variable name="command" select="title" data-type="string"/>

  <xsl:variable name="first_letter" select="substring($command,1,1)" data-type="string"/>
<!-- &lt;H2&gt; -->
<!--   <xsl:value-of select="$first_letter"/> -->
<!-- &lt;/H2&gt; -->

  &lt;A NAME= &quot;<xsl:value-of select="title"/>&quot;&gt;
  &lt;A HREF=
  &quot;cmd<xsl:value-of select="$pos_num"/>.html&quot;
  &gt;&lt;B&gt;<xsl:apply-templates select="title"/>&lt;/B&gt;&lt;/A&gt;
<xsl:apply-templates select="short_description"/>
&lt;BR&gt;

<!-- <xsl:if test="position()*2=last() or position()*2=last()+1"> -->
<!-- &lt;/TD&gt;&lt;TD VALIGN=TOP&gt; -->
<!-- </xsl:if> -->


  </xsl:for-each>

&lt;/DIV&gt;

&lt;/BODY&gt;
&lt;/HTML&gt;
</xsl:document>


<!-- ____ CHAPTERS ______________________________________________ -->

<xsl:variable name="file_toc" select="concat('toc','.html')"/> 
<xsl:document href="{$file_toc}">
  
&lt;HTML&gt;
&lt;HEAD&gt;
&lt;link REL="stylesheet" TYPE="text/css" href="gui.css"&gt;
&lt;TITLE&gt;
  <xsl:apply-templates select="title"/>
&lt;/TITLE&gt;
&lt;/HEAD&gt;

&lt;BODY&gt;

&lt;TABLE WIDTH=100% BORDER=1 CELLPADDING=4 &gt;
&lt;TR&gt;
&lt;TD VALIGN=TOP&gt;

  <xsl:for-each select="manual_parts/part">
    <xsl:sort select="@number" data-type="number"/> 
    <xsl:variable name="part_num" select="@number"/>


<xsl:if test="@number=4">
&lt;/TD&gt;&lt;TD VALIGN=TOP&gt;
</xsl:if>


&lt;DIV&gt;

&lt;H2&gt;
Part <xsl:value-of select="@number"/>
&lt;/H2&gt;

    <xsl:for-each select="chapter"> 
      <xsl:sort select="@number" data-type="number"/>
      <xsl:variable name="chap_num" select="@number"/>

&lt;H3&gt;
  &lt;A NAME=
  &quot;<xsl:value-of select="concat('p',$part_num - 1,'c',$chap_num)"/>&quot;
  &gt;&lt;/A&gt;
Chapter <xsl:value-of select="@number"/>: 
<xsl:apply-templates select="chapter_name"/>
&lt;/H3&gt;  

      <xsl:for-each select="section">
	<xsl:sort select="@number" data-type="number"/>
	<xsl:variable name="sect_num" select="@number"/>


  &lt;A NAME=&quot;<xsl:value-of select="title"/>&quot;&gt;
  &lt;A HREF=&quot;<xsl:value-of select="concat('p',$part_num - 1,'c',$chap_num, 's',$sect_num,'.html')"/>&quot;
  &gt;<xsl:apply-templates select="title"/>&lt;/A&gt;
  &lt;BR&gt;


	</xsl:for-each>
      </xsl:for-each>

&lt;/DIV&gt;
&lt;BR&gt;

    </xsl:for-each>

&lt;/TD&gt;
&lt;/TR&gt;
&lt;/TABLE&gt;

&lt;/BODY&gt;
&lt;/HTML&gt;

  </xsl:document>
  </xsl:template>
  
  <!-- ____  ______________________________________________ -->

  <xsl:template match="title"><xsl:apply-templates/></xsl:template>
  <xsl:template match="description"><xsl:apply-templates/></xsl:template>

  <xsl:template match="commands_and_functions_for">  
   <xsl:variable name="my_type" select="@type"/>
   <xsl:for-each select="/help/cocoa_commands/command[type=$my_type]">
     <xsl:sort select="title"/>
  * <xsl:apply-templates select="title"/> -- <xsl:apply-templates select="short_description"/>
   </xsl:for-each>
  </xsl:template>

  <xsl:template match="coc_version">
    <xsl:value-of select="/help/version/cocoa_version"/>
  </xsl:template>


  <xsl:template match="em">&lt;B&gt;<xsl:value-of select="."/>&lt;/B&gt;</xsl:template>

  <xsl:template match="coc-quotes">&quot;<xsl:apply-templates/>&quot;</xsl:template>


  <xsl:template match="code">&lt;code&gt;<xsl:apply-templates/>&lt;/code&gt;</xsl:template>

  <xsl:template match="lt">&amp;lt;</xsl:template>
  <xsl:template match="less_eq">&amp;lt;=</xsl:template>
  <xsl:template match="tilde">~</xsl:template>
  <xsl:template match="dollar">$</xsl:template>
  <xsl:template match="ocb">{</xsl:template>
  <xsl:template match="ccb">}</xsl:template>
   <xsl:template match="times"> x </xsl:template>
  <xsl:template match="_">_</xsl:template>

  <xsl:template match="ref">&quot;<xsl:apply-templates/>&quot;</xsl:template>

  <xsl:template match="example">
    &lt;h2&gt;example&lt;/h2&gt;
    &lt;pre&gt;
    <xsl:apply-templates/>
    &lt;/pre&gt;
  </xsl:template>

  <xsl:template match="see">
    &lt;h2&gt;See also&lt;/h2&gt;
    &lt;pre&gt;
    <xsl:apply-templates/>
    &lt;/pre&gt;
  </xsl:template>

  <xsl:template match="coc_date">
    <xsl:value-of select="/help/version/date"/>
  </xsl:template>

  <xsl:template match="verbatim">&lt;PRE&gt;<xsl:apply-templates/>&lt;/PRE&gt;</xsl:template>


</xsl:stylesheet> 

