<?xml version="1.0" encoding="ascii"?>

<xsl:stylesheet version="1.1"
		xmlns:xsl="http://www.w3.org/1999/XSL/Transform">


<!--xslt document for creating the man directory for CoCoA-->

  <xsl:output method="text"/>
  <xsl:strip-space elements="*"/>	
  <xsl:preserve-space elements="coc-quotes"/>

  <xsl:template match="help">
\documentclass{mybook}
\ifx\pdfoutput\undefined % if we're not running pdftex  
\else
\usepackage[pdftex,a4paper,colorlinks,hyperindex]{hyperref}
\hypersetup{
  pdfpagelayout=OneColumn,
  pdfstartview      = FitH,%     % La page prend toute la largeur}
%  bookmarks, 
  bookmarksopenlevel = 1,
%  bookmarksnumbered,
%  colorlinks,
%  pagecolor = {green},
  linkcolor = {blue},
  naturalnames,
}
\fi
\usepackage[usenames,dvipsnames]{color}
\usepackage{fancyvrb} % coloured boxes around verbatim region
\usepackage{supertabular} % page breaks in supertabular
\usepackage{amsfonts}

\def\cocoa{{CoCoA}}

\title{\Huge{{\cocoa} 4.3 Manual}}


\setcounter{secnumdepth}{1} 
\setcounter{tocdepth}{1} 

\setlength{\textwidth}{17cm}
\setlength{\oddsidemargin}{-.5cm}
\setlength{\evensidemargin}{-.5cm}

\setlength{\textheight}{24cm}
\setlength{\topmargin}{-1.5cm}

\tolerance=5000

\renewcommand{\thechapter}{\thepart-\arabic{chapter}}

\begin{document}

\maketitle

\tableofcontents



<!-- ===============  MANUAL  =============== -->

% -- PART 1 --------------------------------

  <xsl:for-each select="manual_parts/part">
\part{}
\setcounter{chapter}{0}
    <xsl:sort select="@number" data-type="number"/> 
    <xsl:variable name="part_num" select="@number"/>
    <xsl:for-each select="chapter"> 

% -- CHAPTER --------------------------------
\chapter{<xsl:value-of select="chapter_name"/>}
\label{<xsl:value-of select="chapter_name"/>}

      <xsl:sort select="@number" data-type="number"/>
      <xsl:variable name="chap_num" select="@number"/>
      <xsl:for-each select="section">

% -- SECTION --------------------------------
\section{<xsl:apply-templates select="title"/>}
\label{<xsl:value-of select="title"/>}

	<xsl:sort select="@number" data-type="number"/>
	<xsl:variable name="sect_num" select="@number"/>

        <xsl:apply-templates select="description"/>

      </xsl:for-each>
    </xsl:for-each>
  </xsl:for-each>


<!-- ===============  COMMANDS  =============== -->

% -- NEW PART  --------------------------------
\part{}
\setcounter{chapter}{0}
\chapter{{\cocoa} commands}

  <xsl:for-each select="cocoa_commands/command">
    <xsl:sort select="title"/>
% -- COMMAND --------------------------------
\section{<xsl:apply-templates select="title"/>}
\label{<xsl:value-of select="title"/>}

<!--\subsection*{Syntax}-->
\begin{Verbatim}[frame=single,rulecolor=\color{MidnightBlue},label=syntax]<xsl:value-of select="syntax"/>\end{Verbatim}

\subsection*{Description}
<xsl:apply-templates select="description"/>

<xsl:if test="see">
<!--\subsection*{See Also}-->
\textcolor{OrangeRed}{\large\bf See Also:}
  <xsl:for-each select="see">
      <xsl:apply-templates/> (\ref{<xsl:apply-templates/>} pg.\pageref{<xsl:apply-templates/>})<xsl:if test="not(position()=last())">, </xsl:if>
    </xsl:for-each>
</xsl:if>

  </xsl:for-each>



\end{document}
  </xsl:template>



  <xsl:template match="title"><xsl:apply-templates/></xsl:template>
  <xsl:template match="description"><xsl:apply-templates/></xsl:template>
  <xsl:template match="short_description"><xsl:apply-templates/></xsl:template>

  <xsl:template match="commands_and_functions_for">  
   <xsl:variable name="my_type" select="@type"/>
\begin{center}
\begin{tabular}{ll}
   <xsl:for-each select="/help/cocoa_commands/command[type=$my_type]">
     <xsl:sort select="title"/>
{\verb~<xsl:value-of select="title"/>~} &amp;
      <xsl:apply-templates select="short_description"/>\\
   </xsl:for-each>
\end{tabular}
\end{center}

\noindent
</xsl:template>

  <xsl:template match="coc_version">
    <xsl:value-of select="/help/version/cocoa_version"/>
  </xsl:template>

  <xsl:template match="em">``{\it <xsl:apply-templates/>}''</xsl:template>

  <xsl:template match="coc-quotes">&quot;<xsl:apply-templates/>&quot;</xsl:template>
  <xsl:template match="code" mode="verbatim">&quot;<xsl:apply-templates/>&quot;</xsl:template>  
  <xsl:template match="code">``\verb&amp;<xsl:apply-templates/>&amp;''</xsl:template>

  <xsl:template match="formula">$<xsl:apply-templates/>$</xsl:template>

  <xsl:template match="lt">&lt; </xsl:template>
  <xsl:template match="less_eq">\le </xsl:template>
<!--  <xsl:template match="tilde">$\tilde$</xsl:template> -->
  <xsl:template match="dollar">$</xsl:template>
  <xsl:template match="ocb">{</xsl:template>
  <xsl:template match="ccb">}</xsl:template>
  <xsl:template match="times">\times</xsl:template>
  <xsl:template match="_">\_</xsl:template>

  <xsl:template
  match="example">\begin{Verbatim}[frame=single,rulecolor=\color{PineGreen},label=example]<xsl:apply-templates
  select="." mode="chooseverbatim"/>\end{Verbatim}
</xsl:template>

  <xsl:template match="ref">``<xsl:apply-templates/>'' (\ref{<xsl:apply-templates/>} pg.\pageref{<xsl:apply-templates/>})</xsl:template>

  <xsl:template match="coc_date">
    <xsl:value-of select="/help/version/date"/>
  </xsl:template>

  <xsl:template match="verbatim">\begin{verbatim}<xsl:apply-templates
  select="." mode="chooseverbatim"/>\end{verbatim}</xsl:template>

  <xsl:template match="*" mode="chooseverbatim">
    <xsl:choose>
      <xsl:when test="verbatim='yes'">
	<xsl:apply-templates mode="verbatim"/>
      </xsl:when>
      <xsl:otherwise>
	<xsl:apply-templates/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

</xsl:stylesheet>

