/*
 * Written by Volker Augustin <snowbird@bigfoot.de> (02/2001)
 */

#include "main.h"

void ApplicationWindow::removeCompletionFiles(QStringList paths)
{
	PropertySection *wordCompletionFiles = properties.get(SECTION_WORD_COMPLETION_FILES);

	QStringList::Iterator it;
	for (it = paths.begin(); it != paths.end(); ++it)
	{
		QString path = *it;
		if (wordCompletionFiles->contains(path))
			wordCompletionFiles->remove(path);
	}

	reloadCompletionFiles();
}

void ApplicationWindow::loadCompletionFiles(QStringList paths)
{
	PropertySection *wordCompletionFiles = properties.get(SECTION_WORD_COMPLETION_FILES);

	QStringList::Iterator it;
	for (it = paths.begin(); it != paths.end(); ++it)
	{
		QString path = *it;
		int index = wordCompletionFiles->find(path);
		PropertySectionEntry *entry;
		if (index == -1)
			wordCompletionFiles->append(path, QString::number(LOAD));
		else
		{
			entry = wordCompletionFiles->at(index);
			entry->setValue(QString::number(LOAD));
		}
	}

	reloadCompletionFiles();
}

void ApplicationWindow::unloadCompletionFiles(QStringList paths)
{
	PropertySection *wordCompletionFiles = properties.get(SECTION_WORD_COMPLETION_FILES);

	QStringList::Iterator it;
	for (it = paths.begin(); it != paths.end(); ++it)
	{
		QString path = *it;
		int index = wordCompletionFiles->find(path);
		PropertySectionEntry *entry;
		if (index == -1)
			wordCompletionFiles->append(path, QString::number(DONT_LOAD));
		else
		{
			entry = wordCompletionFiles->at(index);
			entry->setValue(QString::number(DONT_LOAD));
		}
	}

	reloadCompletionFiles();
}

void ApplicationWindow::reloadCompletionFiles()
{
	PropertySection *wordCompletionFiles = properties.get(SECTION_WORD_COMPLETION_FILES);

	/* Remove all previously loaded completions */
	wordCompletions.clear();
	wordFileStateList.clear();

	/* Reload global wordlist */
	QString globalWordCompletions = libraryPath.absFilePath("wordlist.txt");
	WordFileLoadedFlag globalWordFileLoadedFlag = wordCompletions.loadWordCompletions(globalWordCompletions);
	if (globalWordFileLoadedFlag != LOADED)
		qWarning("Could not load global word file \"%s\"!\n", globalWordCompletions.latin1());

	PropertySectionEntry *entry;
	for (entry = wordCompletionFiles->first(); entry != 0; entry = wordCompletionFiles->next())
	{
		QString fileName = entry->key();
		bool ok;
		int load = entry->value().toInt(&ok);
		WordFileLoadFlag loadFlag;
		if (!ok)
			loadFlag = DONT_LOAD;
		else
			loadFlag = (load ? LOAD : DONT_LOAD);

		WordFileLoadedFlag loadedFlag = NOT_LOADED;
		if (loadFlag == LOAD)
			loadedFlag = wordCompletions.loadWordCompletions(fileName);

		WordFileState state(loadFlag, loadedFlag);
		wordFileStateList[fileName] = state;
	}
	emit wordFileListUpdated(wordFileStateList);
}

