<?xml version="1.0" encoding="ascii"?>

<xsl:stylesheet version="1.1"
		xmlns:xsl="http://www.w3.org/1999/XSL/Transform">

  <xsl:preserve-space elements="coc-quotes"/>
  <xsl:preserve-space elements="code"/>

<!--xslt document for creating the man directory for CoCoA-->

  <xsl:output method="text"/>
  <xsl:strip-space elements="*"/>	

  <xsl:template match="help">
  <xsl:for-each select="cocoa_commands/command">
    <xsl:sort select="title"/>
    <xsl:variable name="file" select="concat('cmd',position())"/>
    <xsl:document href="{$file}">
-- <xsl:apply-templates select="title"/>
MEMORY.Doc.Syntax := &quot;<xsl:apply-templates select="syntax"/>&quot;;
MEMORY.Doc.Descr := &quot;<xsl:apply-templates select="description">
</xsl:apply-templates>&quot;;
    </xsl:document>
  </xsl:for-each>
  <xsl:for-each select="manual_parts/part">
    <xsl:sort select="@number" data-type="number"/> 
    <xsl:variable name="part_num" select="@number"/>
    <xsl:for-each select="chapter"> 
      <xsl:sort select="@number" data-type="number"/>
      <xsl:variable name="chap_num" select="@number"/>
      <xsl:for-each select="section">
	<xsl:sort select="@number" data-type="number"/>
	<xsl:variable name="sect_num" select="@number"/>
	<xsl:variable name="file" select="concat('p',$part_num - 1,'c',$chap_num,'s',$sect_num)"/> 
	<xsl:document href="{$file}">
-- <xsl:apply-templates select="title"/>
MEMORY.Doc.Descr := &quot;<xsl:apply-templates select="description">
</xsl:apply-templates>&quot;;
    </xsl:document>
      </xsl:for-each>
    </xsl:for-each>
  </xsl:for-each>
  </xsl:template>

  <xsl:template match="title"><xsl:apply-templates/></xsl:template>
  <xsl:template match="description"><xsl:apply-templates/></xsl:template>

  <xsl:template match="commands_and_functions_for">  
   <xsl:variable name="my_type" select="@type"/>
   <xsl:for-each select="/help/cocoa_commands/command[type=$my_type]">
     <xsl:sort select="title"/>
  * <xsl:apply-templates select="title"/> -- <xsl:apply-templates select="short_description"/>
   </xsl:for-each>
  </xsl:template>

  <xsl:template match="coc_version">
    <xsl:value-of select="/help/version/cocoa_version"/>
  </xsl:template>


  <xsl:template match="em">&quot;&quot;<xsl:value-of select="."/>&quot;&quot;</xsl:template>

  <xsl:template match="coc-quotes">&quot;&quot;<xsl:apply-templates/>&quot;&quot;</xsl:template>


  <xsl:template match="code">&quot;&quot;<xsl:apply-templates/>&quot;&quot;</xsl:template>

  <xsl:template match="lt">&lt;</xsl:template>
  <xsl:template match="less_eq">&lt;=</xsl:template>
  <xsl:template match="tilde">~</xsl:template>
  <xsl:template match="dollar">$</xsl:template>
  <xsl:template match="times"> x </xsl:template>
  <xsl:template match="_">_</xsl:template>

  <xsl:template match="ref">&quot;&quot;<xsl:apply-templates/>&quot;&quot;</xsl:template>

  <xsl:template match="example">-------------- example --------------<xsl:apply-templates/>---------------o-o-o-o---------------</xsl:template>

  <xsl:template match="coc_date">
    <xsl:value-of select="/help/version/date"/>
  </xsl:template>
</xsl:stylesheet> 

