/* $Id: parser.y,v 4.4.1.1 2004/06/18 14:24:25 cocoa Exp $
Parser for CoCoA
-------------------------------------------------------- */

%{

#include <stdlib.h>
#include <string.h>

#include "cocdefs.h"
#include "parser.h"
#include "operator.h"
#include "ca.h"
#include "ck.h"
#include "map.h"
#include "cterm.h"
#include "ctbuf.h"
#include "pipe.h"
#include "utils.h"
#include "machine.h"

extern object math_error;
extern object cocoa_error(char * id);

#ifdef macintosh
int read(int,char*,unsigned); /* is in FCntl.h, not a standard header! */
#ifndef fileno
#define fileno(p)  (p)->_file /* it's a Unix stuff */
#endif
#endif

long LineNo = 1;
int CommentLevel;
char * Ident;
char * Number;
coop Operator;
charbuf StringConst;
long   IntConst;
cterm yycterm;
cterm params[10];

double RealConst;

void yyerror( char * s );
cterm cterm_aux_component(cterm t, ctbuf b);
void cterm_fx_to_f_x(cterm t, cterm * op, cterm *x);
char * cterm_fx_op(cterm t);

char * yy_instr;
int yy_pos;
int yy_len;

%}

	
%start start_cocoa

%token	  ALIAS AMPERSAND AND ASSIGN BLOCK BREAK CARTESIAN CATCH CLEAR COCOLON
	  COCSTR COLON COND DEFINE DELETA DESCRIBE DESTROY DIFFERENT DIGITS DO
	  DOT DOTDOT DOTS DOTVAR ELIF ELIM ELSE END
	  ENDALIAS ENDBLOCK ENDCATCH ENDCOND ENDDEFINE ENDFOR ENDFOREACH
	  ENDIF ENDPACKAGE ENDREPEAT ENDUSING ENDWHILE 
	  EQUAL EXPDIGITS EoF FALSe HELPQUERY
	  FOR FOREACH FRAC FUN_IDENT GREATEROREQUAL HELP IF INDET ISIN In
	  LESSOREQUAL NOT ON OPERATOR_CAST OR PACKAGE PACKAGE_OPERATOR_IDENT
	  PANEL PKG_ID PRINT PRINTLN QUIT RECORD REPEAT RESERVED RETURn
	  RINGASSIGN SET SKIP SOURCE STEP THEN TIMe TO TRUe UNSET UNTIL
	  UPPER_IDENT USE USING VAR WHILE ZMOD ZP

%union
{
	char *		y_string;
	cterm		y_cterm;
	coop		y_operator;
	ctbuf		y_ctbuf;
}

%left ';'
%left ASSIGN

%left '+' '-'
%left '*' FRAC
%left UNARYPREC /* unary minus precedence */
%left DMULT /* direct multiplication (without '*') precedence */
%right '^'

%left OR
%left AND
%left NOT


%type	<y_cterm>	assign_cmd alias_id alias alias_in alias_cmd
			break_cmd bool_expr bool_expr_par bool_term bool_fact bool_atom
			bind command cocstring clear_cmd commands
			coeff_ring_expr catch_cmd cocolon_atom help_query
			cond_expr commands_plus command_plus delete_ring_cmd
			delete_cmd describe_cmd eval_cmd expr exponent
			expdigits expr_atom export expr_par for_cmd fact
			formal_params fun_appl field_id fun_id function
			fun_id2 general_atom varid /*help_cmd*/ int_range
			indet_id indet_decl int_expr if_cmd indet list_expr
			left_expr number nbexpr opt_help option_id print_cmd
			print_on_cmd panels_cmd prg_short_decl prg_long_decl
			package pkg_id program_decl pkg_alias ring_id
			ring_expr ring_zmod ring_decl_cmd ring_modifier
			ring_modifiers return_cmd record_expr repeat_cmd
			start_cocoa skip_cmd source_cmd time_expr term
			use_ring_cmd up_id using_cmd var_ty_id var_id
			while_cmd
	

%type	<y_ctbuf>	alias_list
			bind_list
			cartesian_list cond_else_list command_list
			cmd_plus_list
			expr_list export_list else_list
			var_list
			indet_decl_list int_range_list int_expr_list 
			left_expr_aux
			ring_list  ring_mod_list
			var_ty_list
			 
%type	<y_string>	digits pkg_op_id

%type	<y_operator>	operator_cast
		
%%

start_cocoa	:	command_plus ';' { yycterm=$$;return 0; }
		|	EoF { yycterm=cterm_mk_const(op_EOF);return 0; }
		|	use_ring_cmd ';' { yycterm=$$;return 0; }
		|	alias_in ';' { yycterm=$$;return 0; }
		|	alias_cmd ';' { yycterm=$$;return 0; }
		|	help_query ';' { yycterm=$$;return 0; }
		;

commands_plus	:	cmd_plus_list
			{ if (ctbuf_len($1) == 1) {
			    $$ = ctbuf_lpop($1);
			    ctbuf_free($1); }
                          else $$ = ctbuf_mk_cterm($1,op_COMMANDS); }
		;

cmd_plus_list	:	command_plus
			{ $$ = ctbuf_new($1); }
		|	cmd_plus_list ';' command_plus
			{ $$ = ctbuf_append($1,$3); }
		;

command_plus	:	program_decl
		|	package
		|	command
		;

package		:	PACKAGE pkg_id END
			{ params[0]=$2;
			  params[1]=cterm_mk_const(op_EXPORT);
			  $$=cterm_mk_classic(op_PACKAGE,2,params); }
		|	PACKAGE pkg_id export_list END
			{ params[0]=$2;
			  params[1]=ctbuf_mk_cterm($3,op_EXPORT);
			  $$=cterm_mk_classic(op_PACKAGE,2,params); }
		|	PACKAGE pkg_id pkg_alias export_list END
			{ params[0]=$2;
			  params[1]=ctbuf_mk_cterm($4,op_EXPORT);
			  params[1]=cterm_mk_classic(op_PACKAGE,2,params);
			  params[0]=$3;
			  $$=cterm_mk_classic(op_ALIAS_IN,2,params);
			}
		|	PACKAGE pkg_id ENDPACKAGE
			{ params[0]=$2;
			  params[1]=cterm_mk_const(op_EXPORT);
			  $$=cterm_mk_classic(op_PACKAGE,2,params); }
		|	PACKAGE pkg_id export_list ENDPACKAGE
			{ params[0]=$2;
			  params[1]=ctbuf_mk_cterm($3,op_EXPORT);
			  $$=cterm_mk_classic(op_PACKAGE,2,params); }
		|	PACKAGE pkg_id pkg_alias export_list ENDPACKAGE
			{ params[0]=$2;
			  params[1]=ctbuf_mk_cterm($4,op_EXPORT);
			  params[1]=cterm_mk_classic(op_PACKAGE,2,params);
			  params[0]=$3;
			  $$=cterm_mk_classic(op_ALIAS_IN,2,params);
			}
		;

pkg_alias	:	ALIAS alias_list ';'
			{ $$ = ctbuf_mk_cterm($2,op_ALIASES); }
		;

export_list	:	export ';'
			{ $$ = ctbuf_new($1); }
		|	export_list export ';'
			{ $$ = ctbuf_append($1,$2); }
		;

export		:	program_decl
		;

alias_cmd	:	ALIAS alias_list
			{
			  $$=ctbuf_mk_cterm($2,op_ALIAS_CMD);
			}
		;

alias_in	:	ALIAS alias_list In commands_plus END
			{
			  params[0]=ctbuf_mk_cterm($2,op_ALIASES);
			  params[1]=$4;
			  $$=cterm_mk_classic(op_ALIAS_IN,2,params);
			}
		|	ALIAS alias_list In commands_plus ENDALIAS
			{
			  params[0]=ctbuf_mk_cterm($2,op_ALIASES);
			  params[1]=$4;
			  $$=cterm_mk_classic(op_ALIAS_IN,2,params);
			}
		;

alias_list	:	alias
			{ $$ = ctbuf_new($1); }
		|	alias_list ',' alias
			{ $$ = ctbuf_append($1,$3); }
		;

alias		:	alias_id ASSIGN pkg_id
			{ params[0] = $1;
			  params[1] = $3;
			  $$=cterm_mk_classic(op_ALIAS,2,params); }
		;

pkg_id		:	PKG_ID
			{ params[0]=cterm_mk_NBS(Ident);
			  free_str1(Ident);
			  $$=cterm_mk_classic(op_PKG_ID,1,params); }
		;

commands	:	command_list
			{ if (ctbuf_len($1) == 1) {
			    $$ = ctbuf_lpop($1);
			    ctbuf_free($1); }
                          else $$ = ctbuf_mk_cterm($1,op_COMMANDS); }
		;

command_list	:	command
			{ $$ = ctbuf_new($1); }
		|	command_list ';' command
			{ $$ = ctbuf_append($1,$3); }
		;


command		:	BLOCK commands END { $$=$2; }
		|	BLOCK commands ENDBLOCK { $$=$2; }
		|	eval_cmd
		|	assign_cmd
		|	delete_ring_cmd
		|	delete_cmd
		|	clear_cmd
		|	ring_decl_cmd
		|	describe_cmd
		|	print_cmd
		|	print_on_cmd
		|	skip_cmd
		|	break_cmd
		|	source_cmd
		|	QUIT { $$ = cterm_mk_const(op_QUIT); }
		|	panels_cmd
		|	return_cmd
/* structured commands */
		|	if_cmd
		|	while_cmd
		|	for_cmd
		|	catch_cmd
		|	repeat_cmd
		|	using_cmd
		;

using_cmd	:	USING ring_id DO commands END
			{ params[0]=$2;
			  params[1]=$4;
			  $$=cterm_mk_classic(op_USING,2,params);
			}
		|	USING ring_id DO commands ENDUSING
			{ params[0]=$2;
			  params[1]=$4;
			  $$=cterm_mk_classic(op_USING,2,params);
			}
		;

source_cmd	:	SOURCE expr
			{ params[0]=$2;
			  $$=cterm_mk_classic(op_SOURCE,1,params); }
		|	SOURCE pkg_id
			{ params[0]=$2;
			  $$=cterm_mk_classic(op_SOURCE,1,params); }
		;

time_expr	:	TIMe command
			{ params[0]=$2;
			  $$=cterm_mk_classic(op_TIME,1,params); }
		;

catch_cmd	:	CATCH commands END
			{ params[0]=$2;
			  $$=cterm_mk_classic(op_CATCH,1,params); }
		|	CATCH commands In left_expr END
			{ params[0]=$2;
			  params[1]=$4;
			  $$=cterm_mk_classic(op_CATCH,2,params); }
		|	CATCH commands ENDCATCH
			{ params[0]=$2;
			  $$=cterm_mk_classic(op_CATCH,1,params); }
		|	CATCH commands In left_expr ENDCATCH
			{ params[0]=$2;
			  params[1]=$4;
			  $$=cterm_mk_classic(op_CATCH,2,params); }
		;

return_cmd	:	RETURn expr
			{ params[0]=$2;
			  $$=cterm_mk_classic(op_RETURN,1,params); }
		|	RETURn '(' expr ')'
			{ params[0]=$3;
			  $$=cterm_mk_classic(op_RETURN,1,params); }
		|	RETURn { $$=cterm_mk_const(op_RETURN); }
		;

delete_ring_cmd	:	DESTROY ring_list
			{ $$=ctbuf_mk_cterm($2,op_DELETE_RINGS); }
		;

delete_cmd	:	DELETA var_list
			{ $$=ctbuf_mk_cterm($2,op_DELETE_VARS); }
		;

clear_cmd	:	CLEAR { $$=cterm_mk_const(op_CLEAR); }
		|	CLEAR ring_list { $$=ctbuf_mk_cterm($2,op_CLEAR); }
		;

describe_cmd	:	DESCRIBE expr
			{ params[0]=$2;
			  $$=cterm_mk_classic(op_DESCRIBE,1,params); }
		;

eval_cmd	:	expr
			{ params[0]=$1;
			  $$=cterm_mk_classic(op_EVAL,1,params); }
		;

assign_cmd	:	left_expr ASSIGN expr
			{ params[0]=$1;
			  params[1]=$3;
			  $$=cterm_mk_classic(op_ASSIGN,2,params); }
		;

function	:	AMPERSAND fun_id2 '(' ')'
			{ params[0] = string2cterm("");
			  params[1] = $2;
			  $$ = cterm_mk_classic(op_FUNCTION, 2, params); }
		|	AMPERSAND pkg_op_id '(' ')'
			{ params[0]=
			    string2cterm(my_strtok($2,"."));
			  params[1]=string2cterm(my_strtok(NULL,""));
			  $$ = cterm_mk_classic(op_FUNCTION, 2, params); }
		;

use_ring_cmd	:	USE ring_id
			{ params[0]=$2;
			  $$=cterm_mk_classic(op_SET_CURRENT_RING,1,params); }
		|	USE ring_id RINGASSIGN ring_expr ring_modifiers
			{ params[0]=$2;
			  params[1]=$4;
			  params[2]=$5;
			  $$=cterm_mk_classic(op_DEFINE_AND_USE_RING,3,params);
			}
		|	USE ring_expr ring_modifiers
			{ params[0]=cterm_mk_NBS("CurrentRingEnv");
			  params[0]=cterm_mk_classic(op_VAR,1,params);
			  params[1]=$2;
			  params[2]=$3;
			  $$=cterm_mk_classic(op_DEFINE_AND_USE_RING,3,params);
			}
		;

ring_decl_cmd	:	ring_id RINGASSIGN ring_expr ring_modifiers
			{ params[0]=$1;
			  params[1]=$3;
			  params[2]=$4;
			  $$=cterm_mk_classic(op_DEFINE_RING,3,params); }
		;

print_cmd	:	PRINT expr_list
			{ $$=ctbuf_mk_cterm($2,op_PRINT); }
		|	PRINT { $$=cterm_mk_const(op_PRINT); }
		|	PRINTLN expr_list
			{ $$=ctbuf_mk_cterm($2,op_PRINT_AND_CR); }
		|	PRINTLN { $$=cterm_mk_const(op_PRINT_AND_CR); }
		;

print_on_cmd	:	PRINT expr_list ON expr
			{ params[1]=ctbuf_mk_cterm($2,op_LIST);
			  params[0]=$4;
			  $$=cterm_mk_classic(op_PRINTON,2,params); }
		|	PRINTLN expr_list ON expr
			{ params[1]=ctbuf_mk_cterm($2,op_LIST);
			  params[0]=$4;
			  $$=cterm_mk_classic(op_PRINTON_AND_CR,2,params); }
		|	PRINTLN ON expr
			{ params[0]=$3;
			  params[1]=cterm_mk_const(op_LIST);
			  $$=cterm_mk_classic(op_PRINTON_AND_CR,2,params); }
		;

panels_cmd	:	PANEL up_id
			{ params[0]=$2;
			  $$=cterm_mk_classic(op_PANEL_CMD,1,params); }
		|	PANEL '(' nbexpr ')'
			{ params[0]=$3;
			  $$=cterm_mk_classic(op_PANEL_CMD,1,params); }
		|	PANEL '(' up_id ')'
			{ params[0]=$3;
			  $$=cterm_mk_classic(op_PANEL_CMD,1,params); }
		|	SET option_id
			{ params[0]=$2;
			  params[1]=cterm_mk_const(op_TRUE);
			  $$=cterm_mk_classic(op_SET,2,params); }
		|	SET option_id ASSIGN expr
			{ params[0]=$2;
			  params[1]=$4;
			  $$=cterm_mk_classic(op_SET,2,params); }
		|	UNSET option_id
			{ params[0]=$2;
			  params[1]=cterm_mk_const(op_FALSE);
			  $$=cterm_mk_classic(op_SET,2,params); }
		;

if_cmd		:	IF expr THEN commands else_list END
			{ ctbuf aux_b;
			  aux_b=ctbuf_append($5,$4);
			  aux_b=ctbuf_append(aux_b,$2);
			  $$=ctbuf_mk_cterm_rev(aux_b,op_IF); }
		|	command IF expr
			{
			  params[0]=$3;
			  params[1]=$1;
			  $$=cterm_mk_classic(op_IF,2,params);
			}
		|	IF expr THEN commands else_list ENDIF
			{ ctbuf aux_b;
			  aux_b=ctbuf_append($5,$4);
			  aux_b=ctbuf_append(aux_b,$2);
			  $$=ctbuf_mk_cterm_rev(aux_b,op_IF); }
		;

cond_expr	:	COND expr THEN expr cond_else_list END
			{ ctbuf aux_b;
			  aux_b=ctbuf_append($5,$4);
			  aux_b=ctbuf_append(aux_b,$2);
			  $$=ctbuf_mk_cterm_rev(aux_b,op_COND); }
		|	COND expr THEN expr cond_else_list ENDCOND
			{ ctbuf aux_b;
			  aux_b=ctbuf_append($5,$4);
			  aux_b=ctbuf_append(aux_b,$2);
			  $$=ctbuf_mk_cterm_rev(aux_b,op_COND); }
		;

else_list	:	ELSE commands
			{ $$=ctbuf_new($2); }
		|	ELIF expr THEN commands else_list
			{ $$=ctbuf_append($5,$4);
			  $$=ctbuf_append($$,$2); }
		|	/* empty */
			{ $$=ctbuf_empty(); }
		;

cond_else_list	:	ELSE expr
			{ $$=ctbuf_new($2); }
		|	ELIF expr THEN expr cond_else_list
			{ $$=ctbuf_append($5,$4);
			  $$=ctbuf_append($$,$2); }
		|	/* empty */
			{ $$=ctbuf_empty(); }
		;

while_cmd	:	WHILE expr DO commands END
			{ params[0]=$2;
			  params[1]=$4;
			  $$=cterm_mk_classic(op_WHILE,2,params); }
		|	WHILE expr DO commands ENDWHILE
			{ params[0]=$2;
			  params[1]=$4;
			  $$=cterm_mk_classic(op_WHILE,2,params); }
		;

repeat_cmd	:	REPEAT commands UNTIL expr
			{ params[0]=$2;
			  params[1]=$4;
			  $$=cterm_mk_classic(op_REPEAT,2,params); }
		|	REPEAT commands END
			{ params[0]=$2;
			  params[1]=cterm_mk_const(op_FALSE);
			  $$=cterm_mk_classic(op_REPEAT,2,params); }
		|	REPEAT commands ENDREPEAT
			{ params[0]=$2;
			  params[1]=cterm_mk_const(op_FALSE);
			  $$=cterm_mk_classic(op_REPEAT,2,params); }
		;

for_cmd		:	FOR varid ASSIGN int_expr TO int_expr STEP int_expr
			DO commands END
			{ params[0]=$2;
			  params[1]=$10;
			  params[2]=$4;
			  params[3]=$6;
			  params[4]=$8;
			  $$=cterm_mk_classic(op_FOR,5,params); }
		|	FOR varid ASSIGN int_expr TO int_expr DO commands END
			{ params[0]=$2;
			  params[1]=$8;
			  params[2]=$4;
			  params[3]=$6;
			  $$=cterm_mk_classic(op_FOR,4,params); }
		|	FOREACH varid In expr DO commands END
			{ params[0]=$2;
			  params[1]=$4;
			  params[2]=$6;
			  $$=cterm_mk_classic(op_FOREACH,3,params); }
		|	FOR varid ASSIGN int_expr TO int_expr STEP int_expr
			DO commands ENDFOR
			{ params[0]=$2;
			  params[1]=$10;
			  params[2]=$4;
			  params[3]=$6;
			  params[4]=$8;
			  $$=cterm_mk_classic(op_FOR,5,params); }
		|	FOR varid ASSIGN int_expr TO int_expr 
			DO commands ENDFOR
			{ params[0]=$2;
			  params[1]=$8;
			  params[2]=$4;
			  params[3]=$6;
			  $$=cterm_mk_classic(op_FOR,4,params); }
		|	FOREACH varid In expr DO commands ENDFOREACH
			{ params[0]=$2;
			  params[1]=$4;
			  params[2]=$6;
			  $$=cterm_mk_classic(op_FOREACH,3,params); }
		;

program_decl	:	prg_long_decl
		|	prg_short_decl
		;

prg_long_decl	:	DEFINE fun_id formal_params opt_help commands END
			{ params[0]=$2;
			  params[1]=$3;
			  params[2]=$5;
			  params[3]=cterm_mk_const(op_FALSE); /* long decl */
			  params[4]=$4;
			  $$=cterm_mk_classic(op_PROGRAM,5,params); }
		|	DEFINE fun_id formal_params opt_help commands ENDDEFINE
			{ params[0]=$2;
			  params[1]=$3;
			  params[2]=$5;
			  params[3]=cterm_mk_const(op_FALSE); /* long decl */
			  params[4]=$4;
			  $$=cterm_mk_classic(op_PROGRAM,5,params); }
		;


prg_short_decl	:	fun_appl ASSIGN expr
			{ params[0]=$3;
			  params[2]=cterm_mk_classic(op_RETURN,1,params);
			  cterm_fx_to_f_x($1,&(params[0]),&(params[1]));
			  params[3]=cterm_mk_const(op_TRUE); /* short decl */
			  params[4]=cterm_mk_const(op_NULL_VALUE);
		 	  $$=cterm_mk_classic(op_PROGRAM,5,params); }
		;

opt_help	:	/* empty */ { $$=cterm_mk_const(op_NULL_VALUE); }
		|	HELP cocstring
			{ $$=$2; }
		;

skip_cmd	:	/* empty */ { $$=cterm_mk_const(op_SKIP); }
		|	SKIP { $$=cterm_mk_const(op_SKIP); }
		;

break_cmd	:	BREAK { $$=cterm_mk_const(op_BREAK); }
		;


/* expressions */

int_expr	:	nbexpr
		;

bool_expr	:	bool_term
		|	bool_expr OR bool_term
			{ params[0]=$1;
			  params[1]=$3;
			  $$=cterm_mk_classic(op_OR,2,params); }
/*		|	bool_expr FORALL var_id In expr
		|	EXISTS var_id In expr SUCHTHAT bool_expr */ /* TODO */
		;
		
bool_term	:	bool_fact
		|	bool_term AND bool_fact
			{ params[0]=$1;
			  params[1]=$3;
			  $$=cterm_mk_classic(op_AND,2,params); }
		;

bool_fact	:	bool_atom
		|	NOT bool_fact
			{ params[0]=$2;
			  $$=cterm_mk_classic(op_NOT,1,params); }
		;

bool_atom	:	general_atom
		|	TRUe { $$=cterm_mk_const(op_TRUE); }
		|	FALSe { $$=cterm_mk_const(op_FALSE); }
		|	ring_id COCOLON bool_expr_par
			{ params[0]=$1;
			  params[1]=$3;
			  $$=cterm_mk_classic(op_COCOLON,2,params); }
		|	nbexpr EQUAL nbexpr
			{ params[0]=$1;
			  params[1]=$3;
			  $$=cterm_mk_classic(op_IS_EQ,2,params); }
		|	nbexpr '<' nbexpr
			{ params[0]=$1;
			  params[1]=$3;
			  $$=cterm_mk_classic(op_IS_LT,2,params); }
		|	nbexpr ISIN nbexpr
			{ params[0]=$1;
			  params[1]=$3;
			  $$=cterm_mk_fun_appl(op_IS_IN,2,params); }
		|	nbexpr '>' nbexpr
			{ params[0]=$1;
			  params[1]=$3;
			  $$=cterm_mk_classic(op_IS_GT,2,params); }
		|	nbexpr DIFFERENT nbexpr
			{ params[0]=$1;
			  params[1]=$3;
			  params[0]=cterm_mk_classic(op_IS_EQ,2,params);
			  $$=cterm_mk_classic(op_NOT,1,params); }
		|	nbexpr LESSOREQUAL nbexpr
			{ params[0]=$1;
			  params[1]=$3;
			  $$=cterm_mk_classic(op_IS_LE,2,params); }
		|	nbexpr GREATEROREQUAL nbexpr
			{ params[0]=$1;
			  params[1]=$3;
			  $$=cterm_mk_classic(op_IS_GE,2,params); }
		;

expr		:	bool_expr
		|	nbexpr
		|	time_expr
		|	function
		;

nbexpr		:	term
		|	ELIM '(' expr_list ')'
			{ $$=ctbuf_mk_cterm($3,op_ELIM); }
		|	'@' nbexpr
			{ params[0]=$2;
			  $$=cterm_mk_classic(op_UNLABEL,1,params); }
		|	nbexpr '+' term
			{ params[0]=$1;
			  params[1]=$3;
			  $$ = cterm_mk_classic(op_SUM,2,params); }
		|	nbexpr '-' term 
			{ params[0]=$1;
			  params[1]=$3;
			  $$ = cterm_mk_classic(op_SUBTRACT,2,params); }
		|	term DOTDOT term
			{ params[0]=$1;
			  params[1]=$3;
			  $$=cterm_mk_classic(op_RANGE,2,params); }
		;

cocolon_atom	:	indet
		|	list_expr
		|	fun_appl
		|	expr_par
		;

operator_cast	:	OPERATOR_CAST { $$=Operator;}
		;

term		:	term '*' fact
			{ params[0]=$1;
			  params[1]=$3;
			  $$ = cterm_mk_classic(op_MULT,2,params); }
		|	term FRAC fact
			{ params[0]=$1;
			  params[1]=$3;
			  $$=cterm_mk_classic(op_FRAC,2,params); }
		|	ZMOD fact
			{ params[0]=cterm_mk_NBS("Z");
			  params[0]=cterm_mk_classic(op_VAR,1,params);
			  params[1]=$2;
			  $$=cterm_mk_classic(op_FRAC,2,params); }
		|	term COLON fact
			{ params[0]=$1;
			  params[1]=$3;
			  $$ = cterm_mk_fun_appl2("Colon",2,params); }
		|	cartesian_list
			{ $$ = ctbuf_mk_cterm($1,op_CARTESIAN); }
		|	term fact %prec DMULT
			{ params[0]=$1;
			  params[1]=$2;
			  $$ = cterm_mk_classic(op_MULT,2,params); }
		|	'-' fact %prec UNARYPREC
			{ params[0]=$2;
			  $$ = cterm_mk_classic(op_CHANGE_SIGN,1,params); }
		|	'+' fact %prec UNARYPREC
			{ $$=$2;}
		|	fact
		;

fact		:	expr_atom '^' exponent
			{ params[0]=$1;
			  params[1]=$3;
			  $$=cterm_mk_classic(op_POWER,2,params); }
		|	expr_atom ZP exponent
			{ params[0]=$1;
			  params[1]=$3;
			  $$=cterm_mk_classic(op_ZPMK,2,params); }
		|	expr_atom expdigits /* macintosh */
		  	{ params[0]=$1;
			  params[1]=$2;
			  $$=cterm_mk_classic(op_POWER,2,params); }
		|	cond_expr
		|	expr_atom
		;

exponent	:	expr_atom
		;

expdigits	:	EXPDIGITS
			{ $$=cterm_mk_NBS(Number);
			  free_str(strlen(Number),Number); }
		;

bool_expr_par	:	'(' bool_expr ')'
			{ params[0]=$2;
			  $$=cterm_mk_classic(op_PARENTHESIS,1,params); }
		;

expr_par	:	'(' expr ')'
			{ params[0]=$2;
			  $$=cterm_mk_classic(op_PARENTHESIS,1,params); }
		;

expr_atom	:	number
		|	TRUe { $$=cterm_mk_const(op_TRUE); }
		|	FALSe { $$=cterm_mk_const(op_FALSE); }
		|	RESERVED { 
			  free_str1(Ident); 
			  $$=cterm_mk_const(Operator); }
		|	indet
		|	list_expr
		|	record_expr
		|	ring_id COCOLON cocolon_atom
			{ params[0]=$1;
			  params[1]=$3;
			  $$=cterm_mk_classic(op_COCOLON,2,params); }
		|	general_atom
		|	cocstring
		|	operator_cast list_expr
			{ params[0] = $2;
			  $$ = cterm_mk_classic($1,1,params); }
		;

general_atom	:	left_expr
		|	fun_appl
		|	expr_par
		;


fun_appl	:	pkg_op_id '(' ')' {
			  int aux_n;
			  aux_n=strlen($1);
			  params[0]=
			    cterm_mk_NBS(my_strtok($1,"."));
			  params[1]=cterm_mk_NBS(my_strtok(NULL,""));
			  free_str(aux_n,$1);
			  params[2]=cterm_mk_const(op_LIST);
			  $$=cterm_mk_classic(op_FUN_APPL,3,params); }
		|	pkg_op_id '(' DOTS ')' {
			  int aux_n;
			  aux_n=strlen($1);
			  params[0]=
			    cterm_mk_NBS(my_strtok($1,"."));
			  params[1]=cterm_mk_NBS(my_strtok(NULL,""));
			  free_str(aux_n,$1);
			  params[2]=cterm_mk_const(op_DOTS);
			  $$=cterm_mk_classic(op_FUN_APPL,3,params); }
		|	pkg_op_id '(' expr_list ')' {
			  int aux_n;
			  aux_n=strlen($1);
			  params[0]=
			    cterm_mk_NBS(my_strtok($1,"."));
			  params[1]=cterm_mk_NBS(my_strtok(NULL,""));
			  free_str(aux_n,$1);
			  params[2]=ctbuf_mk_cterm($3,op_LIST);
			  $$=cterm_mk_classic(op_FUN_APPL,3,params); }
		|	fun_id '(' DOTS ')' {
			  params[0]=$1; 
			  params[1]=cterm_mk_const(op_DOTS);
			  $$=cterm_mk_classic(op_FUN_APPL,2,params); }
		|	fun_id '(' expr_list ')'
			{ params[0]=$1;
			  params[1]=ctbuf_mk_cterm($3,op_LIST);
			  $$=cterm_mk_classic(op_FUN_APPL,2,params); }
		|	fun_id '(' ')'
			{ params[0]=$1;
			  params[1]=cterm_mk_const(op_LIST);
			  $$=cterm_mk_classic(op_FUN_APPL,2,params); }
		;

pkg_op_id	:	PACKAGE_OPERATOR_IDENT 
			{ $$=my_strdup(Ident); 
			  free_str1(Ident);
			}
		;

fun_id2		:	fun_id
			{ params[0]=$1; 
			  $$ = cterm_mk_classic(op_STRING,1,params);
			}
		;

fun_id		:	FUN_IDENT
			{ $$=cterm_mk_NBS(Ident); 
			  free_str1(Ident); }
		;

cartesian_list	:	fact CARTESIAN fact
			{ $$=ctbuf_new($1);
			  $$=ctbuf_append($$,$3); }
		|	cartesian_list CARTESIAN fact
			{ $$ = ctbuf_append($1,$3); }
		;

record_expr	:	RECORD '['  ']'
			{ $$ = cterm_mk_const(op_RECORD); }
		|	RECORD '[' bind_list ']'
			{ $$ = ctbuf_mk_cterm($3,op_RECORD); }
		;

bind_list	:	bind
			{ $$ = ctbuf_new($1); }
		|	bind_list ',' bind
			{ $$ = ctbuf_append($1,$3); }
		;

bind		:	field_id EQUAL expr
			{ params[0] = $1;
			  params[0]=cterm_mk_classic(op_VAR,1,params);
			  params[1] = $3;
			  $$=cterm_mk_classic(op_BIND,2,params); }
		|	field_id ASSIGN expr
			{ params[0] = $1;
			  params[0]=cterm_mk_classic(op_VAR,1,params);
			  params[1] = $3;
			  $$=cterm_mk_classic(op_BIND,2,params); }
		;

list_expr	:	'[' ']'
			{ $$ = cterm_mk_const(op_LIST); }
		|	'[' expr_list']'
			{ $$ = ctbuf_mk_cterm($2,op_LIST); }
		|	'[' expr '|' var_id In nbexpr AND bool_expr ']'
			{ params[0]=$4;
			  params[0]=cterm_mk_classic(op_VAR,1,params);
			  params[1]=$6;
			  params[2]=$8;
			  params[3]=$2;
			  $$=cterm_mk_classic(op_ALL,4,params); }
		|	'[' expr '|' var_id In nbexpr ']'
			{ params[0]=$4;
			  params[0]=cterm_mk_classic(op_VAR,1,params);
			  params[1]=$6;
			  params[2]=cterm_mk_const(op_TRUE);
			  params[3]=$2;
			  $$=cterm_mk_classic(op_ALL,4,params); }
		|	'[' var_id In expr '|' bool_expr ']'
			{ cterm aux_params[1];
			  params[0]=$2;
			  params[0]=cterm_mk_classic(op_VAR,1,params);
			  params[1]=$4;
			  params[2]=$6;
			  aux_params[0]=cterm_dup($2);
			  params[3]=cterm_mk_classic(op_VAR,1,aux_params);
			  $$=cterm_mk_classic(op_ALL,4,params); }
		;

/* left expressions */

left_expr	:	var_id
			{ params[0] = $1;
			  $$=cterm_mk_classic(op_VAR,1,params); }
		|	left_expr left_expr_aux
			{ $$=cterm_aux_component($1,$2); }
		|	left_expr DOT field_id
			{ params[0] = $3;
			  params[1]=cterm_mk_classic(op_VAR,1,params);
			  params[0] = $1;
			  $$=cterm_mk_classic(op_RECORDSEL,2,params); }
		|	left_expr DOTVAR '(' expr ')'
			{ params[0] = $4;
			  params[1]=cterm_mk_classic(op_VAR,1,params);
			  params[0] = $1;
			  $$=cterm_mk_classic(op_RECORDSEL,2,params); }
		;

left_expr_aux	:	'[' int_expr_list ']'
			{ $$=$2; }
		|	left_expr_aux '[' int_expr_list ']'
			{ $$=ctbuf_concat_these($1,$3); }
		;

/* lists */

expr_list	:	expr
			{ $$ = ctbuf_new($1); }
		|	expr_list ',' expr
			{ $$ = ctbuf_append($1,$3); }
		;

int_expr_list	:	int_expr
			{ $$ = ctbuf_new($1); }
		|	int_expr_list ',' int_expr
			{ $$ = ctbuf_append($1,$3); }
		;

var_ty_list	:	var_ty_id
			{ $$ = ctbuf_new($1); }
		|	var_ty_list ',' var_ty_id
			{ $$ = ctbuf_append($1,$3); }
		;

var_list	:	varid
			{ $$ = ctbuf_new($1); }
		|	var_list ',' varid
			{ $$ = ctbuf_append($1,$3); }
		;

varid		:	var_id
			{ params[0]=$1; $$=cterm_mk_classic(op_VAR,1,params); }
		;

ring_list	:	ring_id
			{ $$ = ctbuf_new($1); }
		|	ring_list ',' ring_id
			{ $$ = ctbuf_append($1,$3); }
		;


/* identifiers */

alias_id	:	var_id
			{ params[0]=$1; $$=cterm_mk_classic(op_VAR,1,params); }
		;

field_id	:	UPPER_IDENT
			{ $$=cterm_mk_NBS(Ident);
			  free_str1(Ident); }
		|	VAR '(' expr ')'
			{ $$=$3; }
		|	RESERVED
			{ $$=cterm_mk_NBS(Ident);
			  free_str1(Ident); }
		;

up_id		:	UPPER_IDENT
			{ $$=cterm_mk_NBS(Ident);
			  free_str1(Ident); }
		;

var_id		:	up_id
		|	VAR '(' expr ')'
			{ $$=$3; }
		;

ring_id		:	up_id
			{ params[0]=$1;
			  $$=cterm_mk_classic(op_VAR,1,params); }
		|	VAR '(' expr ')'
			{ params[0]=$3;
			  $$=cterm_mk_classic(op_VAR,1,params); }
		;

var_ty_id	:	var_id
		|	VAR var_id
			{ params[0]=$2;
			  $$=cterm_mk_classic(op_VAR,1,params); }
		;

option_id	:	up_id
			{ params[0]=$1;
			  $$=cterm_mk_classic(op_VAR,1,params); }
		;

/* other */

formal_params	:	'(' ')' { $$=cterm_mk_const(op_LIST); }
		|	'(' var_ty_list ')' { $$=ctbuf_mk_cterm($2,op_LIST); }
		|	'(' DOTS ')' { $$=cterm_mk_const(op_DOTS); }
		;

number		:	digits 
			{ $$=cterm_mk_NBS($1);
			  free_str1($1); }
		|	digits DOT digits
			{ char * aux_s;
			  int i;
			  aux_s=my_strsum($1,$3);
			  params[0]=cterm_mk_NBS(aux_s);
			  free_str1(aux_s);
			  free_str1($1);
			  aux_s=malloc_str(strlen($3)+1);
			  aux_s[0]='1';
			  for (i=1;i<=(int)strlen($3);i++) aux_s[i]='0';
			  aux_s[strlen($3)+1]='\0';
			  params[1]=cterm_mk_NBS(aux_s);
			  free_str1(aux_s);
			  free_str1($3);
			  $$=cterm_mk_classic(op_FRAC,2,params); }
		;

digits		:	DIGITS
			{ $$=my_strdup(Number);
			  free_str(strlen(Number),Number); }
		;

indet		:	indet_id
			{ params[0]=$1;
			  $$=cterm_mk_classic(op_INDET,1,params); }
		|	indet_id '[' int_expr_list ']'
			{ ctbuf aux_b;
			  aux_b=ctbuf_new($1);
			  aux_b=ctbuf_concat_these(aux_b,$3);
			  $$=ctbuf_mk_cterm(aux_b,op_INDET); }
		;

coeff_ring_expr	:	ring_id /* Z, ZZ, Q, QQ, D */
/*			{ params[0] = $1;
			  $$=cterm_mk_classic(op_VAR,1,params); } */
		|	ring_zmod
		;			  

ring_expr	:	coeff_ring_expr
		|	coeff_ring_expr '[' indet_decl_list ']'
			{ params[0]=ctbuf_mk_cterm($3,op_INDET_INFO);
			  params[1]=$1;
			  $$=cterm_mk_classic(op_POLY_RING,2,params); }
		;

ring_zmod	:	ZMOD '(' int_expr ')'
			{ params[0]=$3;
			  $$=cterm_mk_classic(op_RING_ZP,1,params); }
		;

ring_modifiers	:	ring_mod_list
			{ $$=ctbuf_mk_cterm($1,op_RING_MODIFIERS); }
		;


ring_mod_list	:	/* empty */
			{ $$=ctbuf_empty(); }
		|	ring_mod_list ',' ring_modifier
			{ $$=ctbuf_append($1,$3); }
		;

ring_modifier	:	nbexpr
		;

indet_decl_list	:	indet_decl
			{ $$ = ctbuf_new($1); }
		|	indet_decl_list indet_decl
			{ $$ = ctbuf_append($1,$2); }
		|	indet_decl_list ',' indet_decl
			{ $$ = ctbuf_append($1,$3); }
		;

indet_decl	:	indet_id
			{ params[0]=$1;
			  $$ = cterm_mk_classic(op_INDET_DECL,1,params); }
		|	indet_id '[' int_range_list ']'
			{ ctbuf aux_b;
			  aux_b=ctbuf_new($1);
			  aux_b=ctbuf_concat_these(aux_b,$3);
			  $$=ctbuf_mk_cterm(aux_b,op_INDET_DECL); }
		;

int_range_list	:	int_range
			{ $$ = ctbuf_new($1); }
		|	int_range_list ',' int_range
			{ $$ = ctbuf_append($1,$3); }
		;

int_range	:	/* int_expr DOTDOT int_expr
			{ params[0]=$1;
			  params[1]=$3;
			  $$=cterm_mk_classic(op_RANGE,2,params); }
 		|	*/ int_expr
		;

indet_id	:	INDET
			{ $$=cterm_mk_NBS(Ident);
			  free_str1(Ident); }
		;

help_query	:	HELPQUERY
			{ params[0]=cterm_mk_NBS(charbuf_buffer(StringConst));
			  charbuf_free(StringConst);
			  $$ = cterm_mk_classic(op_HELPQUERY,1,params); }
		;

cocstring	:	COCSTR
			{ params[0]=cterm_mk_NBS(charbuf_buffer(StringConst));
			  charbuf_free(StringConst);
			  $$ = cterm_mk_classic(op_STRING,1,params); }
		;

%%

#include <stdio.h>

#include "lexical.c"

extern FILE * cocoa_efile;

void yyerror( char * s )
{
  cterm arg[3];
  arg[0]=cterm_mk_NBS(s);
  arg[1]=int2cterm(cdev_curr_line(cdev_parser));
  arg[2]=cterm_mk_NBS(cdev_name(cdev_parser));
  yycterm=cterm_mk_classic(op_TERROR,3,arg);
}


void inityy( void * strBuf )
{
  StringConst=charbuf_init(50,50);
  yy_instr = strBuf;
  yy_pos = 0;
  /* yytext[0] = ' '; */
}


void yy_ii_pp_init( char c )
{
  yy_switch_to_buffer(yy_create_buffer(stdin,YY_BUF_SIZE));
  unput(c);
}

cterm cterm_aux_component(cterm t, ctbuf b)
{ /* E[I,J] -> E[I][J] */
  cterm res_t;
  cterm params[2];
  res_t=t;
  while (! ctbuf_is_empty(b)) {
    params[0]=res_t;
    params[1]=ctbuf_lpop(b);
    res_t=cterm_mk_classic(op_COMPONENT,2,params);
  }
  ctbuf_free(b);
  return res_t;
}

void cterm_remove_var_these(int n, cterm_list params);
void cterm_remove_var_this(cterm * t);

void cterm_fx_to_f_x(cterm t, cterm * op, cterm *x)
{
  /* if t=FUN_APPL("F",[E1,...,En]) op=newId("F"), x=[E1',...,En'] */
  /* if t=F(E1,...,En) op=F, x=[E1',...,En'] */
  /* if t=F(...) op=F, X=... */
  /*   where Ei'=Ei if Ei<>VAR(X) else Ei'=X */

  coop aux_op;
  int pkg_n;
  cterm_list params,aux_params;
  char * n;
  

  aux_op=cterm_op(t);
  pkg_n=op_package(aux_op);
  aux_op=op_index(aux_op);
  params=cterm_params(t);
  if (pkg_n==pkg_builtin_n && aux_op==op_FUN_APPL) {
    *op=params[0];
    aux_params=cterm_params(params[1]);
    cterm_remove_var_these(cterm_arity(params[1]),aux_params);
    *x=params[1];
  } else {
    *op=cterm_mk_NBS(op_map_op_name(pkg_n,aux_op));
    cterm_remove_var_these(cterm_arity(t),params);
    if ((cterm_arity(t)==1) && pkg_n==pkg_builtin_n && 
      (cterm_op(params[0])==op_DOTS))
        *x=cterm_dup(params[0]);
    else
      *x=cterm_mk_classic(op_LIST,cterm_arity(t),params);
  }
  free_cterm_list(cterm_arity(t),params);
  free_cterm(t);
}

char * cterm_fx_op(cterm t)
{
  /* if t=FUN_APPL("F",[E1,...,En]) return F */
  /* if t=F(E1,...,En) Return F */

  coop aux_op;
  int pkg_n;

  aux_op=cterm_op(t);
  pkg_n=op_package(aux_op);
  aux_op=op_index(aux_op);
  if (pkg_n==pkg_builtin_n && aux_op==op_FUN_APPL) 
    return cterm_string(cterm_param(t,0));
  else return op_map_op_name(pkg_n,aux_op);
}

void cterm_remove_var_these(int n, cterm_list params)
{
  int i;
  for (i=0;i<n;i++)
    cterm_remove_var_this(&(params[i]));
}

void cterm_remove_var_this(cterm * t)
{ /* Var("X") -> "X" */
  cterm res_t;
  cterm_list params;
  if (cterm_op(*t)==op_VAR) {
    params=cterm_params(*t);
    res_t=params[0];
    (*t)->params=NULL;
    (*t)->arity=0;
    free_cterm_list(1,params);
    free_cterm(*t);
    *t=res_t;
  } else {
    /* TODO: restituire un errore !!! */
  }
}
